#!/bin/bash
#
# Manage Virtual Machines. Actions:
#  * init: Sets environment config.
#  * launch: Launches client with given data.
#  * term: Cleans environment config.
#
# Usage example:
#  ./vm.sh init br0 10.42.0.1/24
#  ./vm.sh launch 1 br0
#  ./vm.sh term br0

EXIT_CODE=0

# Defaults
BRIDGE="br0"
BRIDGE_IP="10.42.0.1/24"
CLIENT_NUMBER="0"
RAM="512"
IFACE="wlp2s0"

# Functions
init_bridge ()
{
    BRIDGE="${1}"
    BRIDGE_IP="${2}"
    IFACE="${3}"
    
    # Init BRIDGE
    ip link add name ${BRIDGE} type bridge || return $?
    ip addr add ${BRIDGE_IP} dev ${BRIDGE}
    ip link set ${BRIDGE} up
    
    # Enable NAT and IP forwarding
    echo 1 > /proc/sys/net/ipv4/ip_forward
    iptables -t filter -A FORWARD -i ${BRIDGE} -j ACCEPT
    iptables -t nat -A POSTROUTING -o ${IFACE} -j MASQUERADE
    
    # Restart DHCP Server
    # systemctl restart isc-dhcp-server.service
    systemctl restart dhcpd4 \
        || service isc-dhcp-server restart
}

launch_client ()
{
    printf -v CLIENT_NUMBER "%02d" "${1}"
    BRIDGE="${2}"
    TAP="tap${CLIENT_NUMBER}"
    MAC="52:54:00:12:34:${CLIENT_NUMBER}"
    RAM="${3}"
    BINARY="${4}"
    
    # Enable TAP device
    ip tuntap add dev ${TAP} mode tap user $(whoami) || return $?
    # ip link set dev ${IFACE} promisc on
    ip link set dev ${TAP} up
    ip link set dev ${TAP} master ${BRIDGE}
    
    # Start client
    qemu-system-i386 -name "${TAP} - ${MAC}" \
        -device e1000,netdev=net0,mac=${MAC} \
        -netdev tap,id=net0,ifname=${TAP},script=no,downscript=no \
        ${BINARY} -enable-kvm -vga std -m ${RAM}
    
    # Disable TAP device
    ip link set ${TAP} down
    ip link set dev ${TAP} nomaster
    ip tuntap delete dev ${TAP} mode tap
}

term_bridge ()
{
    BRIDGE="${1}"
    
    # Disable NAT and IP forwarding
    echo 0 > /proc/sys/net/ipv4/ip_forward
    iptables -t filter -D FORWARD -i ${BRIDGE} -j ACCEPT
    iptables -t nat -D POSTROUTING -o ${IFACE} -j MASQUERADE
    
    # Delete BRIDGE
    ip link delete ${BRIDGE} type bridge
}

show_help ()
{
    echo "Usage: ${0} init [ BRIDGE=${BRIDGE} ] [ BRIDGE_IP=${BRIDGE_IP} ] [ IFACE=${IFACE} ]"
    echo "       ${0} launch [ CLIENT_NUMBER=${CLIENT_NUMBER} ] [ BRIDGE=${BRIDGE} ] [ RAM=${RAM} ]"
    echo "       ${0} { cdrom | usb } BINARY [ CLIENT_NUMBER=${CLIENT_NUMBER} ] [ BRIDGE=${BRIDGE} ] [ RAM=${RAM} ]"
    echo "       ${0} term [ BRIDGE=${BRIDGE} ]"
    echo "where  CLIENT_NUMBER := 0-99"
}

if [ $(id -u) -eq 0 ]; then
    case "$1" in
        init)
            init_bridge "${2:-${BRIDGE}}" "${3:-${BRIDGE_IP}}" "${4:-${IFACE}}"
            ;;
        launch)
            launch_client "${2:-${CLIENT_NUMBER}}" "${3:-${BRIDGE}}" "${4:-${RAM}}" &
            ;;
        cdrom)
            if [ "${#}" -ge 2 ]; then
                launch_client "${3:-${CLIENT_NUMBER}}" "${4:-${BRIDGE}}" "${5:-${RAM}}" "-cdrom ${2}" &
            else
                show_help
                EXIT_CODE=1
            fi
            ;;
        usb)
            if [ "${#}" -ge 2 ]; then
                launch_client "${3:-${CLIENT_NUMBER}}" "${4:-${BRIDGE}}" "-hda ${2}" &
            else
                show_help
                EXIT_CODE=1
            fi
            ;;
        term)
            term_bridge "${2:-${BRIDGE}}"
            ;;
        *)
            show_help
            EXIT_CODE=1
            ;;
    esac
else
    echo "${0}: Must be run as root"
    EXIT_CODE=1
fi

exit ${EXIT_CODE}
