/**
* Config.js
*
* @description :: TODO: You might write a short summary of how this model works and what it represents here.
* @docs        :: http://sailsjs.org/#!documentation/models
*/

var Promise = require('bluebird'),
    exec = Promise.promisify(require('child_process').exec),
    self = {};

module.exports = _.merge(self, {
    
    attributes: {
        
        /**
         * Config parameter
         */
        key : {
            type: 'string',
            unique: true,
            required: true,
            notNull: true,
            primaryKey: true
        },
            
        /**
         * Config value
         */
        value : {
            type: 'string'
        }
    },
    
    /**
     * Parameters object. Will be injected by `injectConfig`
     */
    _params: {},
    
    /**
     * Inject all config parameters in `_params`
     */
    _injectConfig: function (params) {
        // sails.log.debug('Injecting params...');
        _.merge(self._params, _.reduce(params, function(res, param) {
            res[param.key] = param.value;
            return res;
        }, {}));
    },
    
    /**
     * Load or refresh config parameters
     *
     * @param {Function} cb Callback function returning errors, if any.
     */
    loadConfig: function (cb) {
        self._params = {};
        Config.find(function parametersFound(err, params) {
            if ( err ) {
                return cb(err);
            }
            
            self._injectConfig(params);
            return cb();
        });
    },
    
    /**
     * Set config parameters. Injects new values automatically.
     *
     * @param {Object} paramsDict Parameters to create.
     * @param {Boolean} overwrite Overwrites old values if they are already set.
     * @param {Function} cb Callback function returning errors, if any.
     */
    setParams: function (paramsDict, overwrite, cb) {
        paramsDict = paramsDict || {};
        overwrite = !! overwrite;
        cb = cb || function () { return; };
        
        // sails.log.debug('*** setParams ***');
        // sails.log.debug('paramsDict', paramsDict);
        // sails.log.debug('overwrite', overwrite);
        
        if ( _.isEmpty(paramsDict) ) {
            // sails.log.debug('Empty paramsDict, aborting...');
            return cb();
        }
        
        var paramsArr = _.map(paramsDict, function (value, key) {
            return { key: key, value: value };
        });
        // sails.log.debug('paramsArr', paramsArr);
        
        // sails.log.debug('findOrCreateEach...');
        Config.findOrCreateEach(['key'], paramsArr).exec(function parametersFound(err, params) {
            if ( err ) {
                // sails.log.error('Error!', err);
                return cb(err);
            }
            // sails.log.debug('Found:', params);
            
            if ( overwrite ) {
                // sails.log.debug('Overwriting...');
                _.forEach(params, function (param) {
                    // sails.log.debug(param.value, '!==', paramsDict[param.key], '?');
                    if ( param.value !== paramsDict[param.key] ) {
                        // sails.log.debug('Not equal, updating...');
                        param.value = paramsDict[param.key];
                        param.save();
                    }
                });
            }
            
            // sails.log.debug('Done!');
            self._injectConfig(params);
            return cb();
        });
    },
    
    /**
     * Set one config parameter. An alias of `setParams()`
     *
     * @param {String} key Parameter key
     * @param {String} value Parameter value
     */
    setParam: function (key, value, overwrite, cb) {
        if ( ! key ) {
            // sails.log.debug('Empty param key, aborting...');
            return cb();
        }
        
        var param = {};
        param[key] = value;
        
        return self.setParams(param, overwrite, cb);
    },
    
    /**
     * Helpers to get params, self-explanatories.
     */
    getParam: function (key, def) {
        var _param = self._params[key];
        return typeof _param !== 'undefined' ? _param : def;
    },
    
    allParams: function () {
        return self._params;
    },
    
    /**
     * Status checker.
     */
    getServerStatus: function (next) {
        next = next || function () {};
        var commands = [
            ['DHCP Server', 'ps -A | grep dhcpd > /dev/null 2>&1'],
            ['NFS Server', 'ps -A | grep nfsd > /dev/null 2>&1'],
            // ['NFS Server', 'netstat -tl | grep 2049'],
        ];
        
        return Promise.map(commands, function(command) {
            return exec(command[1]).then(function getStatus (stdout) {
                return [true];
            }).catch(function ignore() {})
            .spread(function setStatus(status) {
                command[1] = !!status;
                return command;
            });
        }).then(function (status) {
            return next(null, status);
        }).catch(function (err) {
            return next(err);
        });
    }
});
