/**
 * Nibium connectors.
 *
 * Define build and iPXE parameters
 *
 * Required fields:
 *  - `build` & `env`: Build binary
 *  - `path`: getfile
 *  - `ipxe`: Build iPXE script
 *  - `nfsroot` & `nfsopts`: /etc/exports
 */

var fs = require('fs'),
    Promise = require('bluebird'),
    fsP = Promise.promisifyAll(require("fs")),
    path = require('path'),
    connsPath = 'connectors',
    connsAbsPath = path.join(__dirname, connsPath);

var Connector = {
    
    /**
     * Loaded connectors
     */
    _connectors: {},
    
    /**
     * Load -or reload- all existing connectors
     */
    load: function (next) {
        var self = this;
        next = next || function () {};
        self._connectors = {};
        
        return fsP.readdirAsync(connsAbsPath).map(function (connector) {
            var connPath = './' + path.join(connsPath, connector);
            var connAbsPath = path.join(__dirname, connPath);
            var connName = path.basename(connector, '.js');
            
            return fsP.statAsync(connAbsPath).then(function (stat) {
                if ( stat.isFile() ) {
                    sails.log.info('Loading connector `' + connPath + '`.');
                    self._connectors[connName] = require(connPath);
                }
            }).catch(function (e) {
                throw e;
            });
        }).then(function () {
            return next(null);
        }).catch(function (err) {
            return next(err);
        });
    },
    
    /**
     * Synchronously load -or reload- all existing connectors
     */
    loadSync: function () {
        var self = this;
        self._connectors = {};
        
        fs.readdirSync(connsAbsPath).forEach(function(connector) {
            var connPath = './' + path.join(connsPath, connector);
            var connAbsPath = path.join(__dirname, connPath);
            var connName = path.basename(connector, '.js');
            
            if ( fs.statSync(connAbsPath).isFile() ) {
                sails.log.info('Loading connector `' + connPath + '`.');
                self._connectors[connName] = require(connPath);
            }
        });
    },
    
    /**
     * Get connector by name
     */
    get: function (connector) {
        var self = this;
        return _.cloneDeep(self._connectors[connector]);
    },
    
    /**
     * Check if connector exists
     */
    exists: function (connector) {
        var self = this;
        return typeof self._connectors[connector] !== 'undefined';
    },
    
    /**
     * Get list of all loaded connectors
     */
    list: function () {
        var self = this;
        return _.keys(self._connectors);
    },
    
};

module.exports = Connector;